---------------------------------------------------------------------------------
-- Navision Price List
---------------------------------------------------------------------------------

drop function nav_import_pricelist ();
create or replace function nav_import_pricelist ()
returns integer as $BODY$
DECLARE
        row			RECORD;
	v_counter		integer;
	v_price_id		integer;
	v_valid_p		integer;

	v_debug			integer;
	v_company_id		integer;
	v_project_id		integer;
	v_task_type_id		integer;
	v_subject_area_id	integer;
	v_source_language_id	integer;
	v_target_language_id	integer;
	v_uom_id		integer;
	v_file_type_id		integer;
	v_difficulty		integer;
	v_final_customer_id	integer;
	v_reuse_band_id		integer;
	v_activity_nr		integer;

	v_valid_from		timestamptz;
	v_valid_to		timestamptz;
	v_currency		varchar;
BEGIN
    v_counter := 0;
    v_debug := 0;

    FOR row IN
        select	*
	from	nav."Activities Price List" l,
		nav."Item" i
	where	l."Activity No_" = i."No_"
    LOOP
	v_counter := v_counter + 1;

	-- Get the company for the entry. At the moment only customers...
	select company_id into v_company_id from im_companies
	where nav_customer_nr = row."Partner No_";
	IF v_company_id is null THEN
		select company_id into v_company_id from im_companies
		where nav_vendor_nr = row."Partner No_";
	END IF;

	-- Get the project for the entries. Project can be NULL
	select project_id into v_project_id from im_projects
	where nav_project_nr = row."Project No_";

	-- Get the final company for the entry.
	select company_id into v_final_customer_id from im_companies
	where nav_customer_nr = row."Customer No_";

	-- Get the Trans Task Type from ActivityGroup/SubActivity
	select category_id into v_task_type_id from im_categories
	where category = row."Activity Group" || '-' || row."Sub-Activity" and category_type = 'Intranet Translation Task Type';

	-- Get the "Segment" (=Subject Area)
	select category_id into v_subject_area_id from im_categories
	where category = row."Segment" and category_type = 'Intranet Translation Subject Area';

	-- Get the "Source Language"
	select category_id into v_source_language_id from im_categories
	where lower(category) = replace(lower(row."Source Language"),' ','_') and category_type = 'Intranet Translation Language';

	-- Get the "Target Language"
	select category_id into v_target_language_id from im_categories
	where lower(category) = replace(lower(row."Target Language"),' ','_') and category_type = 'Intranet Translation Language';

	-- Get the "UoM"
	select category_id into v_uom_id from im_categories
	where aux_string2 = row."Unit of Measure" and category_type = 'Intranet UoM';

	-- Reuse Band
	select category_id into v_reuse_band_id from im_categories
	where aux_string2 = row."Price Type" and category_type = 'Navision Reuse Band';

	v_valid_p := 1;
	IF row."Partner No_" is not null AND v_company_id is null THEN
		RAISE WARNING 'pricelist: company_id is null';
		v_valid_p := 0;
	END IF;
	IF v_uom_id is null THEN
		RAISE WARNING 'pricelist: uom_id is null';
		v_valid_p := 0;
	END IF;
	IF v_task_type_id is null THEN
		RAISE WARNING 'pricelist: task_type_id is null';
		v_valid_p := 0;
	END IF;
	IF row."Segment" is not null AND row."Segment" != '' AND v_subject_area_id is null THEN
		RAISE WARNING 'pricelist: subject_area_id is null';
		v_valid_p := 0;
	END IF;
	IF row."Source Language" is not null AND v_source_language_id is null THEN
		RAISE WARNING 'pricelist: source_language_id is null';
		v_valid_p := 0;
	END IF;
	IF row."Target Language" is not null AND row."Target Language" != 'N/A' AND v_target_language_id is null THEN
		RAISE WARNING 'pricelist: target_language is null';
		v_valid_p := 0;
	END IF;
	IF row."Price Type" is not null AND v_reuse_band_id is null THEN
		RAISE WARNING 'pricelist: reuse_band is null';
		v_valid_p := 0;
	END IF;

	v_valid_from = row."Valid From";
	v_valid_to = row."Valid To";
	IF v_valid_to < '2000-01-01'::date THEN v_valid_to := '2010_12_31'::date; END IF;
	v_currency = coalesce(row."Currency", 'ALL');
	v_difficulty = row."Difficulty";
	v_activity_nr = row."Activity No_";

	IF trim(v_currency) = '' THEN v_currency := 'ALL'; END IF;

	select price_id into v_price_id from im_trans_prices
	where	company_id = v_company_id and
		coalesce(uom_id,0) = coalesce(v_uom_id,0) and
		coalesce(project_id,0) = coalesce(v_project_id,0) and
		coalesce(task_type_id,0) = coalesce(v_task_type_id,0) and
		coalesce(target_language_id,0) = coalesce(v_target_language_id,0) and
		coalesce(source_language_id,0) = coalesce(v_source_language_id,0) and
		coalesce(subject_area_id,0) = coalesce(v_subject_area_id,0) and
		coalesce(file_type_id,0) = coalesce(v_file_type_id,0) and
		coalesce(currency,'AFA') = coalesce(v_currency,'AFA') and
		coalesce(reuse_band_id,0) = coalesce(v_reuse_band_id,0) and
		coalesce(difficulty,-1) = coalesce(v_difficulty,-1) and
		coalesce(activity_nr,-1) = coalesce(v_activity_nr,-1) and
		coalesce(final_customer_id,0) = coalesce(v_final_customer_id,0) and
		coalesce(valid_from,'2000-01-01') = coalesce(v_valid_from,'2000-01-01') and
		coalesce(valid_through,'2000-01-01') = coalesce(v_valid_to,'2000-01-01');


	IF v_debug > 0 OR v_valid_p = 0 THEN
		RAISE NOTICE 'pricelist #%: %=%, %=%, %=%, %=%, %=%, %=%, %=%, %=%, %=%, %=%, %=%, %=%, %=%, %=%', 
		      v_counter,
		      row."Partner No_",  v_company_id,
		      row."Project No_", v_project_id,
		      row."Activity Group" || '-' || row."Sub-Activity", v_task_type_id,
		      row."Unit of Measure", v_uom_id,
		      row."Currency", v_currency,
		      row."Activity No_", v_activity_nr,
		      row."Difficulty", v_difficulty,
		      row."Price Type", v_reuse_band_id,
		      row."Segment", v_subject_area_id,
		      row."Source Language", v_source_language_id,
		      row."Target Language", v_target_language_id,
		      row."Customer No_", v_final_customer_id,
		      row."Valid From"::date, v_valid_from::date,
		      row."Valid To"::date, v_valid_to::date
		;
	END IF;

	-- Insert the price line
	IF v_valid_p > 0 THEN
	    IF v_price_id is null THEN

		INSERT INTO im_trans_prices (
			price_id, uom_id, 
			company_id, task_type_id,
			activity_nr,
			target_language_id, source_language_id, 
			subject_area_id, file_type_id,
			project_id, reuse_band_id, 
			difficulty, final_customer_id,
			currency, price, 
			min_price, note,
 			valid_from, valid_through
		) VALUES (		
			nextval('im_trans_prices_seq'), v_uom_id, 
			v_company_id, v_task_type_id,
			v_activity_nr,
			v_target_language_id, v_source_language_id, 
			v_subject_area_id, v_file_type_id,
			v_project_id, v_reuse_band_id, 
			v_difficulty, v_final_customer_id,
			v_currency, row."Price", 
			NULL, NULL,
			v_valid_from, v_valid_to
		);
	    ELSE
		UPDATE im_trans_prices SET
			uom_id = v_uom_id,
			company_id = v_company_id,
			task_type_id = v_task_type_id,
			activity_nr = v_activity_nr,
			target_language_id = v_target_language_id,
			source_language_id = v_source_language_id,
			subject_area_id = v_subject_area_id,
			file_type_id = v_file_type_id,
			project_id = v_project_id,
			reuse_band_id = v_reuse_band_id,
			difficulty = v_difficulty,
			final_customer_id = v_final_customer_id,
			currency = v_currency,
			price = row."Price",
			min_price = NULL,
			note = NULL,
 			valid_from = v_valid_from,
			valid_through = v_valid_to
		WHERE price_id = v_price_id;
	    END IF;
	END IF;
    END LOOP;

    RETURN 0;
END;$BODY$ language 'plpgsql';
select nav_import_pricelist();



