---------------------------------------------------------------------------------
-- Update the data model
---------------------------------------------------------------------------------

drop trigger im_projects_calendar_update_tr on im_projects;
drop trigger im_forum_topics_calendar_update_tr on im_forum_topics;

-- Projects
alter table im_projects rename column nav_project_no to nav_project_no;
alter table im_projects add nav_project_nr text;
alter table im_projects add bill_to_contact_id integer references persons;
alter table im_projects add nav_delivery_date timestamptz;
alter table im_projects add nav_closing_date timestamptz;
alter table im_projects add nav_last_date timestamptz;

-- Create a check constraint on the project_path file to make
-- sure it is located on the /mnt/... share OR it is a relative path.
-- project_type_id=100 identified Gantt tasks, which are not
-- bound by these restrictions because they do not have a filestorage.
alter table im_projects add constraint im_project_path_ck check (
	project_type_id = 100		OR
	project_path ~ '^/mnt/.*$'	OR
	project_path ~ '^[a-zA-Z0-9_]*$'
);



-- Companies
alter table im_companies add parent_company_id integer;
alter table im_companies rename column nav_customer_no to nav_customer_nr;
alter table im_companies add nav_customer_nr text;
alter table im_companies rename column nav_vendor_no to nav_vendor_nr;
alter table im_companies add nav_vendor_nr text;
alter table im_companies add nav_customer_posting_group text;
alter table im_companies add nav_vendor_posting_group text;
alter table im_companies add nav_pm_percentage numeric(12,1);
alter table im_companies add nav_pm integer references persons;
alter table im_companies add nav_payment_terms integer;
alter table im_companies add nav_currency varchar;

alter table im_companies ADD COLUMN nav_name2 varchar(50);
alter table im_companies ADD COLUMN nav_gl_dimension_1 varchar(20);
alter table im_companies ADD COLUMN nav_blocked smallint;
alter table im_companies ADD COLUMN nav_payment_method integer;
alter table im_companies ADD COLUMN nav_gen_posting_group integer;
alter table im_companies ADD COLUMN nav_vat_posting_group integer;
alter table im_companies ADD COLUMN nav_minimal_tariff_pm numeric(38,20);
alter table im_companies ADD COLUMN nav_minimal_amount numeric(38,20);
alter table im_companies ADD COLUMN nav_division varchar(150);
alter table im_companies ADD COLUMN nav_line_discount_percentage numeric(12,1);

alter table im_companies ALTER COLUMN nav_customer_posting_group TYPE integer USING 
lookup_category_id_from_aux_string2(nav_customer_posting_group::varchar, 'Navision General Posting Group'::varchar);



-- Offices
alter table im_offices rename column nav_customer_no to nav_customer_nr;
alter table im_offices add nav_customer_nr text;
alter table im_offices rename column nav_vendor_no to nav_vendor_nr;
alter table im_offices add nav_vendor_nr text;

-- Persons
alter table persons rename column nav_contact_no to nav_contact_nr;
alter table persons add nav_contact_nr varchar(20);
alter table persons rename column nav_employee_no to nav_employee_nr;
alter table persons add nav_employee_nr varchar(20);
alter table persons add nav_skype varchar;
alter table persons add nav_middle_name varchar;
alter table persons add nav_windows_login varchar;
alter table persons add nav_primary_contact_for_company_id integer;

-- Unique key for Secure Access
alter table persons add sa_object_id integer;
create unique index persons_sa_object_idx on persons (sa_object_id);

-- Fix old DynField definition
update acs_attributes set attribute_name = 'nav_project_nr' where attribute_name = 'navision_project_nr';

-- Costs
alter table im_costs add nav_sales_invoice_nr text;
alter table im_costs add nav_sales_quote_nr text;
alter table im_costs add nav_sales_delnote_nr text;
alter table im_costs add nav_purchase_invoice_nr text;
alter table im_costs add nav_purchase_po_nr text;
alter table im_costs add nav_purchase_receipt_nr text;

-- Categories extension table for Translation Task Types
create table im_categories_trans_task_ext (
	category_id		integer
				constraint im_categories_trans_task_ext_pk
				primary key
				constraint im_categories_trans_task_ext_category_fk
				references im_categories,
	activity_code		integer,
	activity_group		text,
	sub_activity		text,
	reuse_band_id		integer
				constraint im_categories_trans_task_ext_reuse_band_fk
				references im_categories,
	difficulty		integer,
	description		text,
	without_pm_fee_p	integer,
	pm_fee_p		integer	
);



---------------------------------------------------------------------------------
-- Auxilary Functions and Helpers
---------------------------------------------------------------------------------


CREATE OR REPLACE FUNCTION nav.ms_getdate()
RETURNS timestamp without time zone AS 'SELECT LOCALTIMESTAMP'
LANGUAGE 'sql' VOLATILE;


-- Lookup a category in po and return the category_id
create or replace function import_cat (varchar, varchar)
returns integer as '
DECLARE
	p_category	alias for $1;
	p_category_type	alias for $2;
	v_category_id	integer;
BEGIN
	SELECT	c.category_id INTO v_category_id FROM im_categories c
	WHERE	lower(c.category) = lower(p_category)
		AND lower(c.category_type) = lower(p_category_type);

	IF v_category_id is null AND p_category is not null THEN
	   	RAISE WARNING ''import_cat(%,%): Did not find category'', p_category, p_category_type;
	END IF;
	RETURN v_category_id;
END;' language 'plpgsql';
-- select import_cat('open','intranet project status');


-- Lookup the "primary key" (email or username) of a user into a user_id
create or replace function lookup_user_email (varchar, varchar)
returns integer as '
DECLARE
	p_email		alias for $1;
	p_purpose	alias for $2;

	v_user_id	integer;
BEGIN
	IF p_email is null THEN return null; END IF;
	IF p_email = '''' THEN return null; END IF;

	SELECT	p.party_id INTO v_user_id
	FROM	parties p
	WHERE	lower(p.email) = lower(p_email);

	IF v_user_id is null THEN
		SELECT	u.user_id INTO v_user_id
		FROM	users u
		WHERE	lower(u.username) = lower(p_email);
	END IF;

	IF v_user_id is null THEN
	   	RAISE WARNING ''lookup_user_email(%) for %: Did not find user'', p_email, p_purpose;
	END IF;

	RETURN v_user_id;
END;' language 'plpgsql';


-- Lookup the "primary key" (first_names + last_name) of a user into a user_id
create or replace function lookup_user_name (varchar, varchar, varchar)
returns integer as '
DECLARE
	p_first_names	alias for $1;
	p_last_name	alias for $2;
	p_purpose	alias for $3;

	v_user_id	integer;
BEGIN
	IF p_first_names is null OR p_last_name is null THEN return null; END IF;
	IF p_first_names = '''' OR p_last_name = '''' THEN return null; END IF;

	SELECT	p.person_id INTO v_user_id
	FROM	persons p
	WHERE	lower(p.first_names) = lower(p_first_names) and
		lower(p.last_name) = lower(p_last_name);

	IF v_user_id is null THEN
	   	RAISE WARNING ''lookup_user_name(%) for % %: Did not find user'', p_first_names, p_last_name, p_purpose;
	END IF;

	RETURN v_user_id;
END;' language 'plpgsql';




---------------------------------------------------------------------------------
--Helper function im_category_id_from_aux_string2
---------------------------------------------------------------------------------

CREATE OR REPLACE FUNCTION lookup_category_id_from_aux_string2(varchar, varchar) 
RETURNS integer AS $BODY$
DECLARE
	p_aux2		alias for $1;
	p_type		alias for $2;
	v_category_row	RECORD;
BEGIN
	SELECT category_id INTO v_category_row
	FROM im_categories WHERE aux_string2 = p_aux2 AND category_type = p_type
	LIMIT 1;

	IF NOT FOUND THEN
		RAISE WARNING 'lookup_category_id_from_aux_string2(%,%): Did not find category with specified aux_string2', p_aux2, p_type;
		IF p_aux2 IS NULL OR p_aux2 = '' THEN
			RETURN NULL;
		ELSE
			RETURN -1;
		END IF; 
	ELSE
		RETURN v_category_row.category_id;
	END IF;  
END; $BODY$ LANGUAGE 'plpgsql';




-- Function: timesheet_user__get_id(character varying, character varying)
-- DROP FUNCTION timesheet_user__get_id(character varying, character varying);

CREATE OR REPLACE FUNCTION timesheet_user__get_id(character varying, character varying)
  RETURNS integer AS
$BODY$
DECLARE
	p_user_name		alias for $1;
	p_email			alias for $2;
	v_user_id		integer;
BEGIN
	IF coalesce(p_user_name, '') = '' AND coalesce(p_email, '') = '' THEN
		return NULL;
	END IF;

	SELECT o.object_id INTO v_user_id
	FROM acs_objects o, parties pa, persons pe, users u
	WHERE
		o.object_id = pa.party_id AND 
		pa.party_id = pe.person_id AND 
		pe.person_id = u.user_id AND 
		( 
			(coalesce(p_user_name, '') <> '' AND u.username ~~* p_user_name)
			OR (coalesce(p_email, '') <> '' AND pa.email ~~* p_email) 
		)
	ORDER BY 
		pe.sa_object_id;

	IF v_user_id IS NULL THEN
		RAISE WARNING 'timesheet_user__get_id(''%'', ''%'') did not find user.', p_user_name, p_email;
	END IF;

	RETURN v_user_id;
END;$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION timesheet_user__get_id(character varying, character varying) OWNER TO postgres;
